#!python

'''
transform chunks using factorie grammar tagger
'''

import glob
import logging
import os
import random
import subprocess
import time

from streamcorpus import Chunk
from streamcorpus_pipeline.config import configure_logger
from streamcorpus_pipeline.stages import BatchTransform


logger = logging.getLogger(__name__)
configure_logger(logger)


class FactorieBase(object):
    def __init__(self, config):
        self.config = config
        self.scala = config.get('scala', 'scala')
        self.libdirs = config.get('libdirs', [])
        self.classpath_tail = config.get('classpath_tail', [])
        self.memory = config.get('memory', '5G')
        self.cwd = config.get('cwd')  # None is okay for subprocess.Popen(cwd=)
        self.process = None

    def call_factorie(self, inpath, outpath):
        if False:
            self.process = subprocess.Popen(
                [os.path.join(self.cwd, 'factorie.sh'), inpath, outpath],
                cwd=self.cwd)
            return
        cp_parts = filter(None, os.environ.get('CLASSPATH', '').split(':'))
        for libdir in self.libdirs:
            #libdir = os.path.abspath(libdir)
            jars = glob.glob(os.path.join(libdir, '*.jar'))
            jars.sort()
            for jar in jars:
                cp_parts.append(jar)
        cp_parts.extend(self.classpath_tail)
        cmd = [self.scala, '-J-Xmx' + self.memory, 'streamcorpus.factorie.Annotate1', inpath, outpath]
        env = {
            'CLASSPATH': ':'.join(cp_parts),
            'PATH': os.environ['PATH'],
            'PWD': self.cwd,
            'LANG': 'en_US.UTF-8',
        }
        logger.debug('starting %r (env=%r, cwd=%r)', cmd, env, self.cwd)
        self.process = subprocess.Popen(cmd, env=env, cwd=self.cwd)

    def shutdown(self):
        if self.process:
            self.process.terminate()
            self.process = None


class factorie(FactorieBase):
    '''
    incremental transform
    '''
    def __init__(self, config):
        super(FactorieIncremetal, self).__init__(config)

        self.toFactoriePipeName = None
        self.fromFactoriePipeName = None
        self.pipeToFactorie = None
        self.pipeFromFactorie = None
        self.taggedChunkIter = None

    def start(self):
        self.toFactoriePipeName = os.tmpnam()
        self.fromFactoriePipeName = os.tmpnam()
        os.mkfifo(self.toFactoriePipeName)
        os.mkfifo(self.fromFactoriePipeName)
        logger.debug('made fifos %r %r', self.toFactoriePipeName, self.fromFactoriePipeName)

        self.call_factorie(self.toFactoriePipeName, self.fromFactoriePipeName)

        self.pipeToFactorie = Chunk(path=self.toFactoriePipeName, mode='ab')
        self.pipeFromFactorie = Chunk(path=self.fromFactoriePipeName, mode='rb')
        self.taggedChunkIter = iter(self.pipeFromFactorie)

    def close(self):
        self.pipeToFactorie.close()
        self.taggedChunkIter = None
        self.pipeFromFactorie = None
        os.unlink(self.toFactoriePipeName)
        os.unlink(self.fromFactoriePipeName)
        if self.process:
            self.process.terminate()
            self.process = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        return False

    def __call__(self, stream_item, context):
        if not self.process:
            self.start()
        logger.debug('pushing stream item to factorie')
        self.pipeToFactorie.add(stream_item)
        self.pipeToFactorie.flush()
        nc = self.taggedChunkIter.next()
        logger.debug('got item from factorie')
        return nc


def _randstr(length):
    return ''.join(['%x' % random.randint(0,15) for z in xrange(length)])


# multiple inheritance is horrible. sorry.
# I want functionality from FactorieBase, and I think I need this class to be tagged as BatchTransform
class factorie_batch(BatchTransform, FactorieBase):
    "Wrapper for factorie language analysis and tagging"

    def __init__(self, config):
        #BatchTransform.__init__(self, config)
        FactorieBase.__init__(self, config)

    def process_path(self, path):
        '''
        run factorie tagger on streamcorpus chunk file at path
        read data from path, write data out to pathB, move pathB to path
        '''
        start = time.time()
        td = os.path.dirname(path)
        tpath = os.path.join(td, _randstr(40))
        self.call_factorie(path, tpath)
        retcode = self.process.wait()
        if retcode != 0:
            msg = 'factorie returned code %d' % (retcode,)
            logger.error(msg)
            if (not self.config.get('keep_temps', False)) and os.path.exists(tpath):
                logger.info('deleting tempfile %s', tpath)
                try:
                    os.unlink(tpath)
                except:
                    logger.warn('failed to delete tempfile %r', tpath, exc_info=True)
                    # but don't care that much, move on
            raise Exception(msg)
        os.rename(tpath, path)
        logger.info('factorie tagging took %0.1f seconds', time.time() - start)

    def shutdown(self):
        FactorieBase.shutdown(self)


def main():
    import argparse
    from streamcorpus.pipeline.config import load_layered_configs

    global logger
    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    ap = argparse.ArgumentParser()
    ap.add_argument('--config', dest="configs", action="append", help="config .yaml files, may overlap, later values win")
    ap.add_argument('inpath')
    ap.add_argument('outpath')
    args = ap.parse_args()
    
    config = load_layered_configs(args.configs)
    logging.debug('config: %r', config)
    config = config.get('streamcorpus.pipeline', {}).get('factorie_batch', {})
    logging.debug('factorie config: %r', config)

    factorie = FactorieBase(config)
    factorie.call_factorie(args.inpath, args.outpath)
    retcode = factorie.process.wait()
    if retcode != 0:
        logging.info('factorie retcode = %r', retcode)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import os
import sys
import fnmatch
import subprocess

## prepare to run PyTest as a command
from distutils.core import Command

from setuptools import setup, find_packages

from version import get_git_version
VERSION, SOURCE_LABEL = get_git_version()
PROJECT = 'streamcorpus_factorie'
AUTHOR = 'Diffeo, Inc.'
AUTHOR_EMAIL = 'support@diffeo.com'
DESC = 'Tools for building streamcorpus objects for particular collections of text used in TREC KBA.'

def read_file(file_name):
    file_path = os.path.join(
        os.path.dirname(__file__),
        file_name
        )
    return open(file_path).read()

def recursive_glob(treeroot, pattern):
    results = []
    for base, dirs, files in os.walk(treeroot):
        goodfiles = fnmatch.filter(files, pattern)
        results.extend(os.path.join(base, f) for f in goodfiles)
    return results

def recursive_glob_with_tree(treeroot, pattern):
    results = []
    for base, dirs, files in os.walk(treeroot):
        goodfiles = fnmatch.filter(files, pattern)
        one_dir_results = []
        for f in goodfiles:
            one_dir_results.append(os.path.join(base, f))
        results.append((base, one_dir_results))
    return results


def _myinstall(pkgspec):
    subprocess.check_call(['pip', 'install', pkgspec])


class PyTest(Command):
    '''run py.test'''

    description = 'runs py.test to execute all tests'

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        _myinstall('pytest>2.3')
        if self.distribution.install_requires:
            for ir in self.distribution.install_requires:
                _myinstall(ir)

        # reload sys.path for any new libraries installed
        import site
        site.main()
        print sys.path
        # use pytest to run tests
        pytest = __import__('pytest')
        pytest.main(['-s', 'src'])


setup(
    name=PROJECT,
    version=VERSION,
    description=DESC,
    license='MIT/X11 license http://opensource.org/licenses/MIT',
    #long_description=read_file('README.md'),
    source_label=SOURCE_LABEL,
    long_description="",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url='',
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    entry_points= {
        'streamcorpus_pipeline.stages': 'factorie_batch = streamcorpus_factorie:factorie_batch',
    },
    cmdclass={'test': PyTest},
    # We can select proper classifiers later
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',  ## MIT/X11 license http://opensource.org/licenses/MIT
    ],
    install_requires=[
        'streamcorpus>=0.3.5',
        'streamcorpus_pipeline>=0.3.28',
    ],
    data_files = [
        ## this does not appear to actually put anything into the egg...
        ('configs', recursive_glob('configs', '*.yaml')),
    ]
)
